@@ -89,12 +89,14 @@ void bsect_open(char *boot_dev,char *map_file,char *install,int delay,
   int timeout)
 {
     static char coms[] = "0123";
-    static char parity[] = "OoNnEe";
-    static char bps[] = "19200\000300\00038400\0001200\0002400\0004800\000"
-      "9600\000";
+    static char parity[] = "NnOoEe";
+    static char bps[] = 
+        "110\000150\000300\000600\0001200\0002400\0004800\0009600\000"
+        "19200\00038400\00057600\000115200\000?\000?\000?\000?\000"
+        "56000\000";
     GEOMETRY geo;
     struct stat st;
-    int i;
+    int i, speed;
     int i_fd,m_fd,kt_fd,sectors;
     char *message,*colon,*serial,*walk,*this,*keytable,*scheme;
     unsigned char table[SECTOR_SIZE];
@@ -158,33 +160,41 @@ void bsect_open(char *boot_dev,char *map_file,char *install,int delay,
 	if (!*serial || !(this = strchr(coms,*serial)))
 	    die("Invalid serial port in \"%s\" (should be 0-3)",serial);
 	else bsect.par_1.port = (this-coms)+1;
-	if (!serial[1]) bsect.par_1.ser_param = SER_DFL_PRM;
-	else {
+	bsect.par_1.ser_param = SER_DFL_PRM;
+	if (serial[1]) {
 	    if (serial[1] != ',')
 		die("Serial syntax is <port>[,<bps>[<parity>[<bits>]]]");
 	    walk = bps;
-	    bsect.par_1.ser_param = 0;
-	    while (*walk) {
-		if (!strncmp(serial+2,walk,strlen(walk))) break;
-		bsect.par_1.ser_param += 0x20;
-		walk = strchr(walk,0)+1;
+	    speed = 0;
+	    while (*walk && strncmp(serial+2,walk,(i=strlen(walk)))) {
+	        speed++;
+	        walk += i+1;
 	    }
 	    if (!*walk) die("Unsupported baud rate");
-	    if (!*(serial += 2+strlen(walk)))
-		bsect.par_1.ser_param |= SER_DFL_PRM & 0x3f;
-	    else {
+	    bsect.par_1.ser_param &= ~0xE4;
+	    if (speed==16) speed -= 6;  /* convert 56000 to 57600 */
+	    bsect.par_1.ser_param |= ((speed<<5) | (speed>>1)) & 0xE4;
+	    serial += i+2;
+	/* check for parity specified */
+	    if (*serial) {
 		if (!(this = strchr(parity,*serial)))
 		    die("Valid parity values are N, O and E");
-		else bsect.par_1.ser_param |= (((this-parity) >> 1)+1) << 4;
-		if (!serial[1])
-		    bsect.par_1.ser_param |= bsect.par_1.ser_param & 8 ? 2 : 3;
-		else {
+		i = (int)(this-parity)>>1;
+		if (i==2) i++; /* N=00, O=01, E=11 */
+		bsect.par_1.ser_param &= ~(i&1); /* 7 bits if parity specified */
+		bsect.par_1.ser_param |= i<<3;   /* specify parity */
+	/* check if number of bits is there */
+		if (serial[1]) {
 		    if (serial[1] != '7' && serial[1] != '8')
 			die("Only 7 or 8 bits supported");
-		    bsect.par_1.ser_param |= serial[1]-'5';
+		    if (serial[1]=='7')	bsect.par_1.ser_param &= 0xFE;
+		    else bsect.par_1.ser_param |= 0x01;
+		    
 		    if (serial[2]) die("Synax error in SERIAL");
 		}
 	    }
+	    if (verbose>=4) printf("Serial Param = 0x%02X\n", 
+	                                        (int)bsect.par_1.ser_param);
 	}
 	if (delay < 20 && !cfg_get_flag(cf_options,"prompt")) {
 	    fprintf(stderr,"Setting DELAY to 20 (2 seconds)\n");
