@@ -216,6 +216,8 @@ static int next(void)
     }
     if (var && *var) return *var++;
     ch = getc(file);
+    if (ch == '\r')		/* strip DOS <cr> */
+        ch = getc(file);
     if (ch == '\\') {
 	ch = getc(file);
 	if (ch == '$') return ch;
