@@ -108,7 +108,7 @@ typedef struct {
 #define alloc_t(t) ((t *) alloc(sizeof(t)))
 
 
-extern int verbose,test,compact,linear,nowarn,lba32;
+extern int verbose,test,compact,linear,nowarn,lba32,autoauto;
 extern int boot_dev_nr;
 
 extern unsigned short drv_map[DRVMAP_SIZE+1]; /* needed for fixup maps */
