@@ -119,13 +119,26 @@ void do_partition(void)
 }
 
 
-static int has_partitions(dev_t dev)
+int has_partitions(dev_t dev)
 {
-    return MAJOR(dev) == MAJOR_HD || MAJOR(dev) == MAJOR_IDE2 ||
+    if (
+      MAJOR(dev) == MAJOR_HD || MAJOR(dev) == MAJOR_IDE2 ||
       MAJOR(dev) == MAJOR_IDE3 || MAJOR(dev) == MAJOR_IDE4 ||
       MAJOR(dev) == MAJOR_IDE5 || MAJOR(dev) == MAJOR_IDE6 ||
       MAJOR(dev) == MAJOR_XT || MAJOR(dev) == MAJOR_SD ||
-      MAJOR(dev) == MAJOR_ESDI || MAJOR(dev) == MAJOR_DAC960;
+      MAJOR(dev) == MAJOR_SD2 || MAJOR(dev) == MAJOR_SD3 ||
+      MAJOR(dev) == MAJOR_SD4 || MAJOR(dev) == MAJOR_ESDI ||
+      MAJOR(dev) == MAJOR_AMI_HYP ||
+      (MAJOR(dev) >= MAJOR_IDE7 && MAJOR(dev) <= MAJOR_IDE10) ||
+      (MAJOR(dev) >= MAJOR_I2O && MAJOR(dev) <= MAJOR_I2O+7) ||
+      (MAJOR(dev) >= COMPAQ_SMART2_MAJOR && MAJOR(dev) <= COMPAQ_SMART2_MAJOR+7) ||
+      (MAJOR(dev) >= COMPAQ_CISS_MAJOR && MAJOR(dev) <= COMPAQ_CISS_MAJOR+7)
+        ) return 0xFFF0;	/* 4 bit partition mask */
+
+    if (MAJOR(dev) >= MAJOR_DAC960 && MAJOR(dev) <= MAJOR_DAC960+7)
+          return 0xFFF8;	/* 3 bit partition mask */
+
+    return 0;
 }
 
 
@@ -307,10 +320,23 @@ void geo_query_dev(GEOMETRY *geo,int device,int all)
 	case MAJOR_IDE5:
 	    /* fall through */
 	case MAJOR_IDE6:
+	case MAJOR_IDE7:
+	case MAJOR_IDE8:
+	case MAJOR_IDE9:
+	case MAJOR_IDE10:
 	    /* fall through */
 	case MAJOR_ESDI:
 	    /* fall through */
 	case MAJOR_XT:
+	    /* fall through */
+	case MAJOR_I2O:
+	case MAJOR_I2O+1:
+	case MAJOR_I2O+2:
+	case MAJOR_I2O+3:
+	case MAJOR_I2O+4:
+	case MAJOR_I2O+5:
+	case MAJOR_I2O+6:
+	case MAJOR_I2O+7:
 	    geo->device = 0x80+(MINOR(device) >> 6)+(MAJOR(device) == MAJOR_HD ?
 	      0 : last_dev(MAJOR_HD,64));
 	    if (ioctl(fd,HDIO_GETGEO,&hdprm) < 0)
@@ -322,6 +348,9 @@ void geo_query_dev(GEOMETRY *geo,int device,int all)
 	    geo->start = hdprm.start;
 	    break;
 	case MAJOR_SD:
+	case MAJOR_SD2:
+	case MAJOR_SD3:
+	case MAJOR_SD4:
 	    geo->device = 0x80+last_dev(MAJOR_HD,64)+(MINOR(device) >> 4);
 	    if (ioctl(fd,HDIO_GETGEO,&hdprm) < 0)
 		die("geo_query_dev HDIO_GETGEO (dev 0x%04x): %s",device,
@@ -335,6 +364,13 @@ void geo_query_dev(GEOMETRY *geo,int device,int all)
 	    geo->start = hdprm.start;
 	    break;
 	case MAJOR_DAC960:
+	case MAJOR_DAC960+1:
+	case MAJOR_DAC960+2:
+	case MAJOR_DAC960+3:
+	case MAJOR_DAC960+4:
+	case MAJOR_DAC960+5:
+	case MAJOR_DAC960+6:
+	case MAJOR_DAC960+7:
 	    geo->device = 0x80+last_dev(MAJOR_HD,64)+(MINOR(device) >> 3);
 	    if (ioctl(fd,HDIO_GETGEO,&hdprm) < 0)
 		die("geo_query_dev HDIO_GETGEO (dev 0x%04x): %s",device,
@@ -347,6 +383,7 @@ void geo_query_dev(GEOMETRY *geo,int device,int all)
 	    geo->sectors = hdprm.sectors;
 	    geo->start = hdprm.start;
 	    break;
+	case MAJOR_AMI_HYP:
 	case COMPAQ_SMART2_MAJOR+0:
 	case COMPAQ_SMART2_MAJOR+1:
 	case COMPAQ_SMART2_MAJOR+2:
@@ -355,6 +392,14 @@ void geo_query_dev(GEOMETRY *geo,int device,int all)
 	case COMPAQ_SMART2_MAJOR+5:
 	case COMPAQ_SMART2_MAJOR+6:
 	case COMPAQ_SMART2_MAJOR+7:
+	case COMPAQ_CISS_MAJOR+0:
+	case COMPAQ_CISS_MAJOR+1:
+	case COMPAQ_CISS_MAJOR+2:
+	case COMPAQ_CISS_MAJOR+3:
+	case COMPAQ_CISS_MAJOR+4:
+	case COMPAQ_CISS_MAJOR+5:
+	case COMPAQ_CISS_MAJOR+6:
+	case COMPAQ_CISS_MAJOR+7:
 	    geo->device = 0x80+last_dev(MAJOR_HD,64)+(MINOR(device) >> 4);
 	    if (ioctl(fd,HDIO_GETGEO,&hdprm) < 0)
 		die("geo_query_dev HDIO_GETGEO (dev 0x%04x): %s",device,
@@ -405,6 +450,8 @@ int is_first(int device)
 	case MAJOR_ESDI:
 	    /* fall through */
 	case MAJOR_XT:
+	    /* fall through */
+	case MAJOR_I2O:
 	    return MINOR(device) >> 6 ? 0 : !last_dev(MAJOR_HD,64);
 	case MAJOR_SD:
 	    return MINOR(device) >> 4 ? 0 : !last_dev(MAJOR_HD,64);
@@ -422,14 +469,18 @@ void geo_get(GEOMETRY *geo,int device,int user_device,int all)
     int inherited,keep_cyls;
 
     if (MAJOR(device) == MD_MAJOR) {
-        char mdxxx[11];
+        char mdxxx[16];
 	int md_fd, pass;
 	md_array_info_t md_array_info;
 	md_disk_info_t md_disk_info;
 
 	sprintf(mdxxx, "/dev/md%d", MINOR(device));
 	if ((md_fd=open(mdxxx,O_NOACCESS)) < 0)
-	    die("Unable to open %s", mdxxx);
+	{
+	    sprintf(mdxxx, "/dev/md/%d", MINOR(device));
+	    if ((md_fd=open(mdxxx,O_NOACCESS)) < 0)
+		die("Unable to open %s", mdxxx);
+	}
 	if (ioctl(md_fd,GET_ARRAY_INFO,&md_array_info) < 0)
 	    die("Unable to get RAID info on %s", mdxxx);
 	if ((md_array_info.major_version == 0) && (md_array_info.minor_version < 90))
