@@ -29,6 +29,12 @@ typedef struct _dt_entry {
 
 extern DT_ENTRY *disktab;
 
+
+int has_partitions(dev_t dev);
+
+/* indicates that the specified device is a block hard disk device */
+/* returns the partition mask or 0 */
+
 void geo_init(char *name);
 
 /* Loads the disk geometry table. */
