@@ -1,7 +1,7 @@
 /* lilo.c  -  LILO command-line parameter processing */
 /*
 Copyright 1992-1998 Werner Almesberger.
-Copyright 1999-2000 John Coffman.
+Copyright 1999-2001 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -276,8 +276,8 @@ int main(int argc,char **argv)
     while (argc && **++argv == '-') {
 	argc--;
       /* first those options with a mandatory parameter */
-      /* Notably absent are "Rv" */
-	if (strchr("bCdDfiImPrsSuTU", ch=(*argv)[1])) {
+      /* Notably absent are "RuUv" */
+	if (strchr("bCdDfiImPrsST", ch=(*argv)[1])) {
 	    if ((*argv)[2]) param = (*argv)+2;
 	    else {
 		param = *++argv;
@@ -315,7 +315,7 @@ fprintf(stderr,"argc=%d, *argv=%s, ch=%c param=%s\n", argc, *argv, ch, param);
 	    case 'I':
 		identify = param;
 		if (argc) {
-		    ident_opt = *argv++;
+		    ident_opt = *++argv;
 		    argc--;
 		} else {
 		    ident_opt = "i";
@@ -380,12 +380,14 @@ fprintf(stderr,"REBOOT=\"%s\"\n", reboot_arg);
 	    case 'u':
 		validate = 1;
 		/* fall through */
-	    case 'U':
+	    case 'U':	/* argument to -u or -U is optional */
 		uninstall = 1;
-		if (param) {
-		    if (argc) usage(name);
-		    uninst_dev = param;
+		if ((*argv)[2]) param = (*argv)+2;
+		else if (argc && argv[1][0] != '-') {
+		    param = *++argv;
+		    argc--;
 		}
+		uninst_dev = param;
 		break;
 	    case 'v':
 	        if ((*argv)[2]) param = (*argv)+2;
@@ -394,7 +396,7 @@ fprintf(stderr,"REBOOT=\"%s\"\n", reboot_arg);
 	            argc--;
 	        }
 	        if (param) 
-		    verbose = *param - '0';
+		    verbose = to_number(param);
 		else
 	            verbose++;
 		break;
@@ -424,6 +426,8 @@ fprintf(stderr,"REBOOT=\"%s\"\n", reboot_arg);
     if (!new_root) new_root = getenv("ROOT");
     if (new_root && *new_root) {
 	if (chroot(new_root) < 0) die("chroot %s: %s",new_root,strerror(errno));
+	if (chdir("/dev") < 0 && !nowarn)
+	    fprintf(stderr, "Warning: root at %s has no /dev directory\n", new_root);
 	if (chdir("/") < 0) die("chdir /: %s",strerror(errno));
     }
     if (atexit(temp_remove)) die("atexit() failed");
@@ -436,7 +440,7 @@ fprintf(stderr,"REBOOT=\"%s\"\n", reboot_arg);
 	}
 	printf(", Copyright (C) 1992-1998 Werner Almesberger\n"
 	       "Linux Real Mode Interface library Copyright (C) 1998 Josh Vanderhoof\n"
-	       "Development beyond version 21 Copyright (C) 1999-2000 John Coffman\n"
+	       "Development beyond version 21 Copyright (C) 1999-2001 John Coffman\n"
 	       );
         if (verbose>0) printf("Released %s and compiled at %s on %s.\n",
 	        VERSION_DATE, __TIME__, __DATE__);
@@ -547,8 +551,8 @@ fprintf(stderr,"REBOOT=\"%s\"\n", reboot_arg);
 	    disktab = disk;
 	    if (cfg_get_strg(cf_options,"boot")) cfg_unset(cf_options,"boot");
 	    if (cfg_get_strg(cf_options,"map")) cfg_unset(cf_options,"map");
-	    strncpy(new_name,dev.name,MAX_TOKEN);
-	    new_name[MAX_TOKEN] = '\0';
+	    strncpy(new_name,dev.name,8);
+	    new_name[8] = '\0';
 	    cfg_set(cf_options,"boot",new_name,NULL);
 	    snprintf(new_name,MAX_TOKEN,"%s.%04x",md_boot_map,device);
 	    cfg_set(cf_options,"map",new_name,NULL);
@@ -592,7 +596,8 @@ fprintf(stderr,"REBOOT=\"%s\"\n", reboot_arg);
             cfg_init(cf_top);
             if (cfg_parse(cf_top)) cfg_error("Syntax error");
 	}
-	if (!bsect_number()) die("No images have been defined.");
+	if (!bsect_number())
+	    die("No images have been defined or default image doesn't exist.");
 	check_fallback();
 	if (!test)
 	    if (cfg_get_strg(cf_options,"force-backup"))
