@@ -32,16 +32,26 @@
 #define MAJMIN_RAM	0x101 /* RAM disk */
 #define MAJOR_FD	2 /* floppy disks */
 #define MAJOR_HD	3 /* IDE-type hard disks */
-#define MAJOR_SD	8 /* SCSI disks */
+#define MAJOR_SD	8 /* SCSI disks 0-15 */
+#define MAJOR_SD2	65 /* SCSI disks 16-31 */
+#define MAJOR_SD3	66 /* SCSI disks 32-47 */
+#define MAJOR_SD4	67 /* SCSI disks 48-63 */
 #define MAJOR_XT	13 /* XT-type hard disks */
 #define MAJOR_IDE2	22 /* IDE on second interface */
 #define MAJOR_IDE3	33 /* IDE on third interface */
 #define MAJOR_IDE4	34 /* IDE on fourth interface */
 #define MAJOR_ESDI	36 /* PS/2 ESDI drives */
 #define MAJOR_DAC960	48 /* First Mylex DAC960 PCI RAID controller */
-#define MAJOR_IDE5	55 /* IDE on fifth interface */
+#define MAJOR_IDE5	56 /* IDE on fifth interface */
 #define MAJOR_IDE6	57 /* IDE on sixth interface */
-#define COMPAQ_SMART2_MAJOR	72 /* First Smart/2 Major */
+#define MAJOR_IDE7	88 /* IDE on seventh interface */
+#define MAJOR_IDE8	89 /* IDE on eighth interface */
+#define MAJOR_IDE9	90 /* IDE on ninth interface */
+#define MAJOR_IDE10	91 /* IDE on tenth interface */
+#define MAJOR_I2O	80  /* First I2O block device 80-87 */
+#define COMPAQ_SMART2_MAJOR	72 /* First Smart/2 Major 72-79 */
+#define COMPAQ_CISS_MAJOR	104 /* First CCISS Major 104-111 */
+#define MAJOR_AMI_HYP	101 /* AMI Hyper Disk RAID controller */
 
 #define MAX_IMAGES      ((SECTOR_SIZE*2-2)/sizeof(IMAGE_DESCR))
 			  /* maximum number of images */
@@ -271,4 +281,4 @@ CL_POINTER	= 0x228			! new pointer is dword address
 
 #endif
 
-#endif
\ No newline at end of file
+#endif
